"""
TextTransform - assorted text transformations (e.g. rot13)
Copyright (C) 2010  Brian S. Stephan

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import base64
import re

from Module import Module

class TextTransform(Module):
    """
    Do a number of text transformations, like rot13.
    """

    def do(self, connection, event, nick, userhost, what, admin_unlocked):
        """
        Pass the real work on to one of our methods and see if any bite.
        """

        # if the caller replied, this will be the reply and they'll have returned true
        reply = ['']

        if self.rot13(what, reply):
            return self.irc.reply(event, reply[0])
        elif self.base64(what, reply):
            return self.irc.reply(event, reply[0])
        elif self.upper(what, reply):
            return self.irc.reply(event, reply[0])
        elif self.lower(what, reply):
            return self.irc.reply(event, reply[0])
        elif self.al_bhed(what, reply):
            return self.irc.reply(event, reply[0])

    def rot13(self, what, reply):
        """
        Apply a rot13 method to the text if first word is 'rot13'.
        """

        match = re.search('^!rot13\s+(.*)$', what)
        if match:
            text = match.group(1)
            reply[0] = text.encode('rot13', 'ignore')
            return True

    def base64(self, what, reply):
        """
        Encode/decode base64 string.
        """

        match = re.search('^!base64\s+encode\s+(.*)$', what)
        if match:
            text = match.group(1)
            reply[0] = base64.encodestring(text).replace('\n','')
            return True

        match = re.search('^!base64\s+decode\s+(.*)$', what)
        if match:
            text = match.group(1)
            reply[0] = base64.decodestring(text).replace('\n','')
            return True

    def upper(self, what, reply):
        """
        Convert a string to uppercase.
        """

        match = re.search('^!upper\s+(.*)$', what)
        if match:
            text = match.group(1)
            reply[0] = text.upper()
            return True

    def lower(self, what, reply):
        """
        Convert a string to lowercase.
        """

        match = re.search('^!lower\s+(.*)$', what)
        if match:
            text = match.group(1)
            reply[0] = text.lower()
            return True

    def al_bhed(self, what, reply):
        """
        Convert a string to al bhed.
        """

        cipher = {'A':'Y','B':'P','C':'L','D':'T','E':'A','F':'V','G':'K','H':'R','I':'E','J':'Z','K':'G','L':'M','M':'S','N':'H','O':'U','P':'B','Q':'X','R':'N','S':'C','T':'D','U':'I','V':'J','W':'F','X':'Q','Y':'O','Z':'W'}
        decipher = dict([(v,k) for (k,v) in cipher.iteritems()])

        match = re.search('^!al-bhed\s+(.*)$', what)
        if match:
            text = match.group(1)
            trans = []
            for i in range(len(text)):
                if text[i] in cipher:
                    trans.append(cipher[text[i]])
                elif text[i].upper() in cipher:
                    trans.append(cipher[text[i].upper()].lower())
                else:
                    trans.append(text[i])

            reply[0] = "".join(trans)
            return True

        match = re.search('^!deal-bhed\s+(.*)$', what)
        if match:
            text = match.group(1)
            trans = []
            for i in range(len(text)):
                if text[i] in decipher:
                    trans.append(decipher[text[i]])
                elif text[i].upper() in cipher:
                    trans.append(decipher[text[i].upper()].lower())
                else:
                    trans.append(text[i])

            reply[0] = "".join(trans)
            return True

# vi:tabstop=4:expandtab:autoindent
# kate: indent-mode python;indent-width 4;replace-tabs on;
